package org.example.basictrekking;

/**
 * Created by Cristina on 09/11/2015.
 */

import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.Toast;
import android.widget.ToggleButton;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;


public class MapaTrack extends FragmentActivity implements GoogleMap.OnMapClickListener, GoogleMap.OnInfoWindowClickListener{
    private final LatLng UPV = new LatLng(39.481106, -0.340987);

    public static GoogleMap mapa;
    private String nombreTrack;

    private static WaypointsDB WaypointsDB;
    private static PolylineOptions track;
    private static boolean deMarcadores = false;
    private static boolean deActualizacion = false;

    private ToggleButton MarkerButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.mapatrack);

        mapa = ((SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map)).getMap();
        // Comprobamos que tenemos una referencia correcta del mapa
        if(mapa != null) {

            mapa.setMapType(GoogleMap.MAP_TYPE_SATELLITE);

            mapa.setMyLocationEnabled(true);
            mapa.getUiSettings().setZoomControlsEnabled(true);
            mapa.getUiSettings().setCompassEnabled(true);

            mapa.moveCamera(CameraUpdateFactory.newLatLngZoom(UPV, 14));
        }

        WaypointsDB = new WaypointsDB(this);

        mapa.setOnMapClickListener(this);
        mapa.setOnInfoWindowClickListener(this);

        MarkerButton = (ToggleButton) findViewById(R.id.marcadores);
        MarkerButton.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    mostrarMarkers(null);
                } else {
                    ocultarMarkers(null);
                }
            }
        });

        //Dibujamos el track
        actualizartrack();

    }
    @Override
    protected void onResume(){
        super.onResume();
    }

    @Override
    protected void onPause(){
        super.onPause();
    }

    void actualizartrack()
    {
        LatLng waypoint;
        track = new PolylineOptions();

        SQLiteDatabase bd = WaypointsDB.getReadableDatabase();

        //En cursor se almacenarán todos los datos que tengan el nombre del track seleccionado
        Cursor cursor = bd.rawQuery("SELECT * FROM waypoints WHERE nombre = '" + Tracks.nombreTracklista +"'", null);
        //Nos movemos al primer elemento del track seleccionado
        if(cursor.moveToFirst()) {
            //Centramos el mapa en el primer waypoint del track y lo representamos
            if(deMarcadores == false || deActualizacion == false) {
                mapa.animateCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(cursor.getDouble(2), cursor.getDouble(3)), 14));
            }
            deMarcadores = false;
            deActualizacion = false;
            waypoint = new LatLng(cursor.getDouble(2), cursor.getDouble(3));
            track.add(waypoint);
            mapa.addMarker(new MarkerOptions()
                    .position(new LatLng(cursor.getDouble(2), cursor.getDouble(3))).title(getString(R.string.puntoInicio))
                    .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW)));

            // Recorremos el cursor para dibujar el resto del track
            while (cursor.moveToNext()) {
                waypoint = new LatLng(cursor.getDouble(2), cursor.getDouble(3));
                if (waypoint != null) {
                    track.add(waypoint);
                }
            }
            track.color(Color.YELLOW);
            track.width(5);
            mapa.addPolyline(track);
        }
        else
            Toast.makeText(this, R.string.noWaypoints, Toast.LENGTH_SHORT).show();

        //Cerramos el cursor y la base de datos
        cursor.close();
        bd.close();
    }

    void mostrarMarkers(View view)
    {
        String foto = "foto";
        String titulo;
        int indiceTitulo;
        double latitud = 0.0;
        double longitud = 0.0;

        PhotosDB photosDB = new PhotosDB(this);
        SQLiteDatabase db = photosDB.getReadableDatabase();
        Cursor cursor = db.rawQuery("SELECT * FROM photos WHERE track = '" + Tracks.nombreTracklista + "'", null);

        if(cursor.moveToFirst()) {
            foto = cursor.getString(2);
            // Extraemos el nombre de la foto para utilizarla en el marcador
            indiceTitulo = foto.indexOf("files/");
            titulo = foto.substring(indiceTitulo + 6);
            latitud = cursor.getDouble(3);
            longitud = cursor.getDouble(4);


            mapa.addMarker(new MarkerOptions()
                    .position(new LatLng(latitud, longitud)).title(String.valueOf(latitud) + "," + String.valueOf(longitud))
                    .snippet(titulo)
                    .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN)));


            while (cursor.moveToNext()) {
                foto = cursor.getString(2);
                indiceTitulo = foto.indexOf("files/");
                titulo = foto.substring(indiceTitulo + 6);
                latitud = cursor.getDouble(3);
                longitud = cursor.getDouble(4);

                mapa.addMarker(new MarkerOptions()
                        .position(new LatLng(latitud, longitud)).title(String.valueOf(latitud) + "," + String.valueOf(longitud))
                        .snippet(titulo)
                        .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN)));

            }
        }else{
            Toast.makeText(this, R.string.noMarkers, Toast.LENGTH_SHORT).show();
            //Deshabilitamos de nuevo el botón de marcadores, ya que no hay
            MarkerButton.setChecked(false);
        }
        //Cerramos el cursor y la base de datos
        cursor.close();
        db.close();
    }


    void ocultarMarkers(View view)
    {
        mapa.clear();
        deMarcadores = true;
        actualizartrack();
    }

    @Override
    public void onMapClick(LatLng latLng) {
        mapa.clear();
        deActualizacion = true;
        actualizartrack();
        //Si los marcadores están seleccionados, hay que mostrarlos
        if(MarkerButton.isChecked())
            mostrarMarkers(null);
    }

    @Override
    public void onInfoWindowClick(Marker marker) {
        String FICHERO =  Environment.getExternalStorageDirectory() + "/Android/data/org.example.basictrekking/files/";
        //Mostramos la foto en la Galería
        Intent intent = new Intent(android.content.Intent.ACTION_VIEW);
        Uri data = Uri.parse("file://"+ FICHERO + marker.getSnippet());
        intent.setDataAndType(data, "image/*");
        startActivity(intent);
    }
}

